package com.pmease.quickbuild.plugin.tracker.redmine;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import javax.ws.rs.core.MediaType;

import org.apache.commons.httpclient.HttpStatus;
import org.dom4j.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.pmease.quickbuild.plugin.tracker.core.TrackerException;
import com.pmease.quickbuild.plugin.tracker.core.extensionpoint.Issue;
import com.pmease.quickbuild.plugin.tracker.core.utils.RestClientFactory;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;

public class RedmineCli {
	private static final Logger logger = LoggerFactory.getLogger(RedmineCli.class);
	
	private final String host;
	private final String username;
	private final String password;
	private final String projectId;
	
	public RedmineCli(String username, String password, String host, String projectId) {
		this.host = host;
		this.projectId = projectId;
		this.username = username;
		this.password = password;
	}

	public static RedmineCli newInstance() {
		RedmineSetting setting = RedmineSetting.get();
		return new RedmineCli(setting.getUser(), setting.getPassword(), setting.getHost(), setting.getAssociatedProject());
	}
	
	String getIssueDocument(String id) {
		logger.debug("Retriving issue " + id);
		
		WebResource resource = resource().path("projects/" + projectId + "/issues/" + id + ".xml");
		
		ClientResponse response = null;
		try {
			response = resource.accept(MediaType.TEXT_XML).get(ClientResponse.class);
		} catch (Exception e) {
			throw new TrackerException("Unable to fetch issue from remote server. ", e);
		}
		
		int status = response.getStatus();
		
		if (status == HttpStatus.SC_OK) {
			return response.getEntity(String.class);
		} else if (status == HttpStatus.SC_NOT_FOUND) {
			return null;
		} else {
			throw new TrackerException("Fetch issue " + id + " failed. " + response);
		}
	}
	
	public Issue getIssue(String id) {
		String doc = getIssueDocument(id);
		return RedmineHelper.parseIssue(doc);
	}
	
	public Issue createIssue(Map<String, String> options) {
		Document doc = RedmineHelper.mapToIssueDocument(options);
		ClientResponse response = resource()
					.path("/projects/" + projectId + "/issues.xml")
					.type(MediaType.TEXT_XML)
					.post(ClientResponse.class, doc.asXML());
		
		int status = response.getStatus();
		String entity = response.getEntity(String.class);
		if (status == HttpStatus.SC_CREATED) {
			return RedmineHelper.parseIssue(entity);
		} else {
			throw new TrackerException("Create issue failed. " + response);
		}
	}
	
	public List<String> getProjects() {
		return Collections.emptyList();
	}
	
	public void getUsers() {
		ClientResponse response = resource()
					.path("/users.xml")
					.queryParam("login", "steve")
					.type(MediaType.TEXT_XML).get(ClientResponse.class);
		
		if (response.getStatus() == HttpStatus.SC_OK) {
			System.out.println(response.getEntity(String.class));
		} else {
			throw new TrackerException("Fetch users failed. " + response);
		}
	}
	
	WebResource resource() {
		return RestClientFactory.resource(host, username, password);
	}
}
